@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package JSON::Schema::Modern; # git description: v0.572-4-g8f5216ff
+package JSON::Schema::Modern; # git description: v0.573-18-g6f7f97bf
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate data against a schema
 # KEYWORDS: JSON Schema validator data validation structure specification
 
-our $VERSION = '0.573';
+our $VERSION = '0.574';
 
 use 5.020;  # for fc, unicode_strings features
 use Moo;
@@ -237,6 +237,7 @@ sub traverse ($self, $schema_reference, $config_override = {}) {
     effective_base_uri => Mojo::URL->new(''),
     errors => [],
     identifiers => [],
+    subschemas => [],
     configs => {},
     callbacks => $config_override->{callbacks} // {},
     evaluator => $self,
@@ -452,6 +453,8 @@ sub _traverse_subschema ($self, $schema, $state) {
   return E($state, 'EXCEPTION: maximum traversal depth (%d) exceeded', $self->max_traversal_depth)
     if $state->{depth}++ > $self->max_traversal_depth;
 
+  push $state->{subschemas}->@*, $state->{traversed_schema_path}.$state->{schema_path};
+
   my $schema_type = get_type($schema);
   return 1 if $schema_type eq 'boolean';
 
@@ -724,7 +727,7 @@ sub add_vocabulary ($self, $classname) {
   }
 }
 
-# $schema uri => [ spec_version, [ vocab classes ] ].
+# $schema uri => [ spec_version, [ vocab classes, in evaluation order ] ].
 has _metaschema_vocabulary_classes => (
   is => 'bare',
   isa => HashRef[
@@ -737,25 +740,25 @@ has _metaschema_vocabulary_classes => (
   lazy => 1,
   default => sub {
     my @modules = map use_module('JSON::Schema::Modern::Vocabulary::'.$_),
-      qw(Core Applicator Validation FormatAnnotation Content MetaData Unevaluated);
+      qw(Core Validation FormatAnnotation Applicator Content MetaData Unevaluated);
     +{
       'https://json-schema.org/draft/2020-12/schema' => [ 'draft2020-12', [ @modules ] ],
       do { pop @modules; () },
       'https://json-schema.org/draft/2019-09/schema' => [ 'draft2019-09', \@modules ],
-      'http://json-schema.org/draft-07/schema#' => [ 'draft7', \@modules ],
+      'http://json-schema.org/draft-07/schema' => [ 'draft7', \@modules ],
     },
   },
 );
 
-sub _get_metaschema_vocabulary_classes { $_[0]->__metaschema_vocabulary_classes->{$_[1]} }
-sub _set_metaschema_vocabulary_classes { $_[0]->__metaschema_vocabulary_classes->{$_[1]} = $mvc_type->($_[2]) }
+sub _get_metaschema_vocabulary_classes { $_[0]->__metaschema_vocabulary_classes->{$_[1] =~ s/#$//r} }
+sub _set_metaschema_vocabulary_classes { $_[0]->__metaschema_vocabulary_classes->{$_[1] =~ s/#$//r} = $mvc_type->($_[2]) }
 sub __all_metaschema_vocabulary_classes { values $_[0]->__metaschema_vocabulary_classes->%* }
 
 # retrieves metaschema info either from cache or by parsing the schema for vocabularies
 # throws a JSON::Schema::Modern::Result on error
 sub _get_metaschema_info ($self, $metaschema_uri, $for_canonical_uri) {
   # check the cache
-  my $metaschema_info = $self->__metaschema_vocabulary_classes->{$metaschema_uri};
+  my $metaschema_info = $self->_get_metaschema_vocabulary_classes($metaschema_uri);
   return @$metaschema_info if $metaschema_info;
 
   # otherwise, fetch the metaschema and parse its $vocabulary keyword.
@@ -1016,7 +1019,7 @@ JSON::Schema::Modern - Validate data against a schema
 
 =head1 VERSION
 
-version 0.573
+version 0.574
 
 =head1 SYNOPSIS
 
