@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::Applicator;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Applicator vocabulary
 
-our $VERSION = '0.573';
+our $VERSION = '0.574';
 
 use 5.020;
 use Moo;
@@ -39,15 +39,16 @@ sub evaluation_order { 3 }
 # - properties and patternProperties must be evaluated before additionalProperties
 # - in-place applicators and properties, patternProperties, additionalProperties must be evaluated
 #   before unevaluatedProperties (in the Unevaluated vocabulary)
-# - contains must be evaluated before maxContains, minContains (in the Validator vocabulary)
-sub keywords ($self, $spec_version) {
+# - contains must be evaluated before maxContains, minContains (implemented here, rather than in the Validation vocabulary)
+sub keywords ($class, $spec_version) {
   return (
     qw(allOf anyOf oneOf not if then else),
     $spec_version eq 'draft7' ? 'dependencies' : 'dependentSchemas',
     $spec_version !~ qr/^draft(7|2019-09)$/ ? 'prefixItems' : (),
     'items',
     $spec_version =~ qr/^draft(7|2019-09)$/ ? 'additionalItems' : (),
-    qw(contains properties patternProperties additionalProperties propertyNames),
+    'contains', $spec_version ne 'draft7' ? qw(maxContains minContains) : (),
+    qw(properties patternProperties additionalProperties propertyNames),
     $spec_version eq 'draft2019-09' ? qw(unevaluatedItems unevaluatedProperties) : (),
   );
 }
@@ -68,10 +69,10 @@ foreach my $phase (qw(traverse eval)) {
 
 sub _traverse_keyword_allOf { shift->traverse_array_schemas(@_) }
 
-sub _eval_keyword_allOf ($self, $data, $schema, $state) {
+sub _eval_keyword_allOf ($class, $data, $schema, $state) {
   my @invalid;
   foreach my $idx (0 .. $schema->{allOf}->$#*) {
-    if ($self->eval($data, $schema->{allOf}[$idx], +{ %$state,
+    if ($class->eval($data, $schema->{allOf}[$idx], +{ %$state,
         schema_path => $state->{schema_path}.'/allOf/'.$idx })) {
     }
     else {
@@ -88,11 +89,11 @@ sub _eval_keyword_allOf ($self, $data, $schema, $state) {
 
 sub _traverse_keyword_anyOf { shift->traverse_array_schemas(@_) }
 
-sub _eval_keyword_anyOf ($self, $data, $schema, $state) {
+sub _eval_keyword_anyOf ($class, $data, $schema, $state) {
   my $valid = 0;
   my @errors;
   foreach my $idx (0 .. $schema->{anyOf}->$#*) {
-    next if not $self->eval($data, $schema->{anyOf}[$idx],
+    next if not $class->eval($data, $schema->{anyOf}[$idx],
       +{ %$state, errors => \@errors, schema_path => $state->{schema_path}.'/anyOf/'.$idx });
     ++$valid;
     last if $state->{short_circuit};
@@ -105,10 +106,10 @@ sub _eval_keyword_anyOf ($self, $data, $schema, $state) {
 
 sub _traverse_keyword_oneOf { shift->traverse_array_schemas(@_) }
 
-sub _eval_keyword_oneOf ($self, $data, $schema, $state) {
+sub _eval_keyword_oneOf ($class, $data, $schema, $state) {
   my (@valid, @errors);
   foreach my $idx (0 .. $schema->{oneOf}->$#*) {
-    next if not $self->eval($data, $schema->{oneOf}[$idx],
+    next if not $class->eval($data, $schema->{oneOf}[$idx],
       +{ %$state, errors => \@errors, schema_path => $state->{schema_path}.'/oneOf/'.$idx });
     push @valid, $idx;
     last if @valid > 1 and $state->{short_circuit};
@@ -127,8 +128,8 @@ sub _eval_keyword_oneOf ($self, $data, $schema, $state) {
 
 sub _traverse_keyword_not { shift->traverse_subschema(@_) }
 
-sub _eval_keyword_not ($self, $data, $schema, $state) {
-  return 1 if not $self->eval($data, $schema->{not},
+sub _eval_keyword_not ($class, $data, $schema, $state) {
+  return 1 if not $class->eval($data, $schema->{not},
     +{ %$state, schema_path => $state->{schema_path}.'/not',
       short_circuit_suggested => 1, # errors do not propagate upward from this subschema
       collect_annotations => 0,     # nor do annotations
@@ -141,10 +142,10 @@ sub _traverse_keyword_if { shift->traverse_subschema(@_) }
 sub _traverse_keyword_then { shift->traverse_subschema(@_) }
 sub _traverse_keyword_else { shift->traverse_subschema(@_) }
 
-sub _eval_keyword_if ($self, $data, $schema, $state) {
+sub _eval_keyword_if ($class, $data, $schema, $state) {
   return 1 if not exists $schema->{then} and not exists $schema->{else}
     and not $state->{collect_annotations};
-  my $keyword = $self->eval($data, $schema->{if},
+  my $keyword = $class->eval($data, $schema->{if},
      +{ %$state, schema_path => $state->{schema_path}.'/if',
         short_circuit_suggested => !$state->{collect_annotations},
         errors => [],
@@ -152,21 +153,21 @@ sub _eval_keyword_if ($self, $data, $schema, $state) {
     ? 'then' : 'else';
 
   return 1 if not exists $schema->{$keyword};
-  return 1 if $self->eval($data, $schema->{$keyword},
+  return 1 if $class->eval($data, $schema->{$keyword},
     +{ %$state, schema_path => $state->{schema_path}.'/'.$keyword });
   return E({ %$state, keyword => $keyword }, 'subschema is not valid');
 }
 
 sub _traverse_keyword_dependentSchemas { shift->traverse_object_schemas(@_) }
 
-sub _eval_keyword_dependentSchemas ($self, $data, $schema, $state) {
+sub _eval_keyword_dependentSchemas ($class, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
   foreach my $property (sort keys $schema->{dependentSchemas}->%*) {
     next if not exists $data->{$property};
 
-    if ($self->eval($data, $schema->{dependentSchemas}{$property},
+    if ($class->eval($data, $schema->{dependentSchemas}{$property},
         +{ %$state, schema_path => jsonp($state->{schema_path}, 'dependentSchemas', $property) })) {
       next;
     }
@@ -179,7 +180,7 @@ sub _eval_keyword_dependentSchemas ($self, $data, $schema, $state) {
   return 1;
 }
 
-sub _traverse_keyword_dependencies ($self, $schema, $state) {
+sub _traverse_keyword_dependencies ($class, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'object');
 
   my $valid = 1;
@@ -197,13 +198,13 @@ sub _traverse_keyword_dependencies ($self, $schema, $state) {
     }
     else {
       # as in dependentSchemas
-      $valid = 0 if not $self->traverse_property_schema($schema, $state, $property);
+      $valid = 0 if not $class->traverse_property_schema($schema, $state, $property);
     }
   }
   return $valid;
 }
 
-sub _eval_keyword_dependencies ($self, $data, $schema, $state) {
+sub _eval_keyword_dependencies ($class, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
@@ -219,7 +220,7 @@ sub _eval_keyword_dependencies ($self, $data, $schema, $state) {
     }
     else {
       # as in dependentSchemas
-      if ($self->eval($data, $schema->{dependencies}{$property},
+      if ($class->eval($data, $schema->{dependencies}{$property},
           +{ %$state, schema_path => jsonp($state->{schema_path}, 'dependencies', $property) })) {
         next;
       }
@@ -237,31 +238,31 @@ sub _traverse_keyword_prefixItems { shift->traverse_array_schemas(@_) }
 
 sub _eval_keyword_prefixItems { goto \&_eval_keyword__items_array_schemas }
 
-sub _traverse_keyword_items ($self, $schema, $state) {
+sub _traverse_keyword_items ($class, $schema, $state) {
   if (is_plain_arrayref($schema->{items})) {
     return E($state, 'array form of "items" not supported in %s', $state->{spec_version})
       if $state->{spec_version} !~ /^draft(?:7|2019-09)$/;
 
-    return $self->traverse_array_schemas($schema, $state);
+    return $class->traverse_array_schemas($schema, $state);
   }
 
-  $self->traverse_subschema($schema, $state);
+  $class->traverse_subschema($schema, $state);
 }
 
-sub _eval_keyword_items ($self, $data, $schema, $state) {
+sub _eval_keyword_items ($class, $data, $schema, $state) {
   goto \&_eval_keyword__items_array_schemas if is_plain_arrayref($schema->{items});
   goto \&_eval_keyword__items_schema;
 }
 
 sub _traverse_keyword_additionalItems { shift->traverse_subschema(@_) }
 
-sub _eval_keyword_additionalItems ($self, $data, $schema, $state) {
+sub _eval_keyword_additionalItems ($class, $data, $schema, $state) {
   return 1 if not exists $state->{_last_items_index};
   goto \&_eval_keyword__items_schema;
 }
 
 # prefixItems (draft 2020-12), array-based items (all drafts)
-sub _eval_keyword__items_array_schemas ($self, $data, $schema, $state) {
+sub _eval_keyword__items_array_schemas ($class, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
   return 1 if ($state->{_last_items_index}//-1) == $data->$#*;
 
@@ -277,7 +278,7 @@ sub _eval_keyword__items_array_schemas ($self, $data, $schema, $state) {
         _schema_path_suffix => $idx, collect_annotations => $state->{collect_annotations} & ~1 },
         'item not permitted');
     }
-    elsif ($self->eval($data->[$idx], $schema->{$state->{keyword}}[$idx],
+    elsif ($class->eval($data->[$idx], $schema->{$state->{keyword}}[$idx],
         +{ %$state, data_path => $state->{data_path}.'/'.$idx,
           schema_path => $state->{schema_path}.'/'.$state->{keyword}.'/'.$idx,
           collect_annotations => $state->{collect_annotations} & ~1 })) {
@@ -297,7 +298,7 @@ sub _eval_keyword__items_array_schemas ($self, $data, $schema, $state) {
 }
 
 # schema-based items (all drafts), and additionalItems (up to and including draft2019-09)
-sub _eval_keyword__items_schema ($self, $data, $schema, $state) {
+sub _eval_keyword__items_schema ($class, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
   return 1 if ($state->{_last_items_index}//-1) == $data->$#*;
 
@@ -311,7 +312,7 @@ sub _eval_keyword__items_schema ($self, $data, $schema, $state) {
         exists $schema->{prefixItems} || $state->{keyword} eq 'additionalItems' ? 'additional ' : '');
     }
     else {
-      if ($self->eval($data->[$idx], $schema->{$state->{keyword}},
+      if ($class->eval($data->[$idx], $schema->{$state->{keyword}},
         +{ %$state, data_path => $state->{data_path}.'/'.$idx,
           schema_path => $state->{schema_path}.'/'.$state->{keyword},
           collect_annotations => $state->{collect_annotations} & ~1 })) {
@@ -333,14 +334,14 @@ sub _eval_keyword__items_schema ($self, $data, $schema, $state) {
 
 sub _traverse_keyword_contains { shift->traverse_subschema(@_) }
 
-sub _eval_keyword_contains ($self, $data, $schema, $state) {
+sub _eval_keyword_contains ($class, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
 
   $state->{_num_contains} = 0;
   my (@errors, @valid);
 
   foreach my $idx (0 .. $data->$#*) {
-    if ($self->eval($data->[$idx], $schema->{contains},
+    if ($class->eval($data->[$idx], $schema->{contains},
         +{ %$state, errors => \@errors,
           data_path => $state->{data_path}.'/'.$idx,
           schema_path => $state->{schema_path}.'/contains',
@@ -365,9 +366,43 @@ sub _eval_keyword_contains ($self, $data, $schema, $state) {
     : A($state, @valid == @$data ? true : \@valid);
 }
 
+# 'maxContains' and 'minContains' are owned by the Validation vocabulary, but do nothing if the
+# Applicator vocabulary is omitted and depend on the result of 'contains', so they are implemented
+# here, to be evaluated after 'contains'
+
+sub _traverse_keyword_maxContains { 1 }
+
+sub _eval_keyword_maxContains ($class, $data, $schema, $state) {
+  return 1 if not grep $_ eq 'JSON::Schema::Modern::Vocabulary::Validation',
+    $state->{vocabularies}->@*;
+
+  return 1 if not exists $state->{_num_contains};
+  return 1 if not is_type('array', $data);
+
+  return E($state, 'array contains more than %d matching items', $schema->{maxContains})
+    if $state->{_num_contains} > $schema->{maxContains};
+
+  return 1;
+}
+
+sub _traverse_keyword_minContains { 1 }
+
+sub _eval_keyword_minContains ($class, $data, $schema, $state) {
+  return 1 if not grep $_ eq 'JSON::Schema::Modern::Vocabulary::Validation',
+    $state->{vocabularies}->@*;
+
+  return 1 if not exists $state->{_num_contains};
+  return 1 if not is_type('array', $data);
+
+  return E($state, 'array contains fewer than %d matching items', $schema->{minContains})
+    if $state->{_num_contains} < $schema->{minContains};
+
+  return 1;
+}
+
 sub _traverse_keyword_properties { shift->traverse_object_schemas(@_) }
 
-sub _eval_keyword_properties ($self, $data, $schema, $state) {
+sub _eval_keyword_properties ($class, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
@@ -382,7 +417,7 @@ sub _eval_keyword_properties ($self, $data, $schema, $state) {
         _schema_path_suffix => $property }, 'property not permitted');
     }
     else {
-      if ($self->eval($data->{$property}, $schema->{properties}{$property},
+      if ($class->eval($data->{$property}, $schema->{properties}{$property},
           +{ %$state, data_path => jsonp($state->{data_path}, $property),
             schema_path => jsonp($state->{schema_path}, 'properties', $property),
             collect_annotations => $state->{collect_annotations} & ~1 })) {
@@ -399,18 +434,18 @@ sub _eval_keyword_properties ($self, $data, $schema, $state) {
   return 1;
 }
 
-sub _traverse_keyword_patternProperties ($self, $schema, $state) {
+sub _traverse_keyword_patternProperties ($class, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'object');
 
   my $valid = 1;
   foreach my $property (sort keys $schema->{patternProperties}->%*) {
     $valid = 0 if not assert_pattern({ %$state, _schema_path_suffix => $property }, $property);
-    $valid = 0 if not $self->traverse_property_schema($schema, $state, $property);
+    $valid = 0 if not $class->traverse_property_schema($schema, $state, $property);
   }
   return $valid;
 }
 
-sub _eval_keyword_patternProperties ($self, $data, $schema, $state) {
+sub _eval_keyword_patternProperties ($class, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
@@ -424,7 +459,7 @@ sub _eval_keyword_patternProperties ($self, $data, $schema, $state) {
           _schema_path_suffix => $property_pattern }, 'property not permitted');
       }
       else {
-        if ($self->eval($data->{$property}, $schema->{patternProperties}{$property_pattern},
+        if ($class->eval($data->{$property}, $schema->{patternProperties}{$property_pattern},
             +{ %$state, data_path => jsonp($state->{data_path}, $property),
               schema_path => jsonp($state->{schema_path}, 'patternProperties', $property_pattern),
               collect_annotations => $state->{collect_annotations} & ~1 })) {
@@ -444,7 +479,7 @@ sub _eval_keyword_patternProperties ($self, $data, $schema, $state) {
 
 sub _traverse_keyword_additionalProperties { shift->traverse_subschema(@_) }
 
-sub _eval_keyword_additionalProperties ($self, $data, $schema, $state) {
+sub _eval_keyword_additionalProperties ($class, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
@@ -461,7 +496,7 @@ sub _eval_keyword_additionalProperties ($self, $data, $schema, $state) {
         'additional property not permitted');
     }
     else {
-      if ($self->eval($data->{$property}, $schema->{additionalProperties},
+      if ($class->eval($data->{$property}, $schema->{additionalProperties},
           +{ %$state, data_path => jsonp($state->{data_path}, $property),
             schema_path => $state->{schema_path}.'/additionalProperties',
             collect_annotations => $state->{collect_annotations} & ~1 })) {
@@ -480,12 +515,12 @@ sub _eval_keyword_additionalProperties ($self, $data, $schema, $state) {
 
 sub _traverse_keyword_propertyNames { shift->traverse_subschema(@_) }
 
-sub _eval_keyword_propertyNames ($self, $data, $schema, $state) {
+sub _eval_keyword_propertyNames ($class, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
   foreach my $property (sort keys %$data) {
-    if ($self->eval($property, $schema->{propertyNames},
+    if ($class->eval($property, $schema->{propertyNames},
         +{ %$state, data_path => jsonp($state->{data_path}, $property),
           schema_path => $state->{schema_path}.'/propertyNames',
           collect_annotations => $state->{collect_annotations} & ~1 })) {
@@ -514,7 +549,7 @@ JSON::Schema::Modern::Vocabulary::Applicator - Implementation of the JSON Schema
 
 =head1 VERSION
 
-version 0.573
+version 0.574
 
 =head1 DESCRIPTION
 
